@@ -1,215 +1,27 @@
-get.pop.observed <- function(pop.pred, country, sex=c('both', 'male', 'female'), age='all') {
-	sex <- match.arg(sex)
-	data <- pop.pred$inputs$pop.matrix
-	if(sex == 'both') {
-		data <- data[['male']][,colnames(data[['male']])] + data[['female']][,colnames(data[['male']])]
-	} else data <- data[[sex]]
-	country.idx <- grep(paste('^', country, '_', sep=''), rownames(data), value=FALSE)
-	data <- data[country.idx,]
-	if(age[1]=='psr')  # potential support ratio
-		return(colSums(data[get.psr.nominator.index(),])/colSums(data[get.psr.denominator.startindex():nrow(data),]))
-	age.idx <- if(age[1]=='all') 1:nrow(data) else age
-	return(colSums(data[age.idx,]))
-}
 
-get.psr.nominator.index <- function() return(5:13)
-get.psr.denominator.startindex <- function() return(14)
-
-get.pop.trajectories <- function(pop.pred, country, sex=c('both', 'male', 'female'), age='all',
- 									nr.traj=NULL, typical.trajectory=FALSE) {
-	traj.file <- file.path(pop.pred$output.directory, paste('totpop_country', country, '.rda', sep=''))
-	quant <- hch <- age.idx <- traj <- traj.idx <-  NULL
-	load.traj <- is.null(nr.traj) || nr.traj > 0
-	if (!file.exists(traj.file)) 
-		return(list(trajectories=traj, index=traj.idx, quantiles=quant, age.idx=age.idx, half.child=hch))
-	load(traj.file)
-	sex <- match.arg(sex)
-	max.age <- dim(totp)[1] # should be 27
-	age.idx <- if(age[1]=='all') 1:max.age else age
-	if(sex == 'both' && age[1]=='all') {
-		if(load.traj) traj <- totp
-		quant <- pop.pred$quantiles
-		hch <- totp.hch
-	} else {
-		if (age[1] == 'psr') { # potential support ratio
-			if(sex == 'both' && load.traj)
-				traj <- (colSums(totpm[get.psr.nominator.index(),,,drop=FALSE]) + 
-								colSums(totpf[get.psr.nominator.index(),,,drop=FALSE]))/(
-								colSums(totpm[get.psr.denominator.startindex():max.age,,,drop=FALSE]) + 
-									colSums(totpf[get.psr.denominator.startindex():max.age,,,drop=FALSE]))
-			if(sex == 'male' && load.traj) 
-				traj <- colSums(totpm[get.psr.nominator.index(),,,drop=FALSE])/colSums(
-											totpm[get.psr.denominator.startindex():max.age,,,drop=FALSE])
-			if(sex == 'female' && load.traj) 
-				traj <- colSums(totpf[get.psr.nominator.index(),,,drop=FALSE])/colSums(
-											totpf[get.psr.denominator.startindex():max.age,,,drop=FALSE])
-		} else {
-			if(sex == 'both') {
-				if(load.traj) traj <- colSums(totpm[age.idx,,,drop=FALSE]) + colSums(totpf[age.idx,,,drop=FALSE])
-				hch <- colSums(totpm.hch[age.idx,,,drop=FALSE]) + colSums(totpf.hch[age.idx,,,drop=FALSE])
-			} else {
-				if(sex=='male') {
-					if(load.traj) traj <- colSums(totpm[age.idx,,,drop=FALSE])
-					hch <- colSums(totpm.hch[age.idx,,,drop=FALSE])
-					if (length(age.idx) == max.age) quant <- pop.pred$quantilesM
-					else {if (length(age.idx) == 1) quant <- pop.pred$quantilesMage[,age.idx,,]}
-				} else { # female
-					if(load.traj) traj <- colSums(totpf[age.idx,,,drop=FALSE])
-					hch <- colSums(totpf.hch[age.idx,,,drop=FALSE])
-					if (length(age.idx) == max.age) quant <- pop.pred$quantilesF
-					else {if (length(age.idx) == 1) quant <- pop.pred$quantilesFage[,age.idx,,]}
-				}
-			}
-		}
-	}
-	if(load.traj) {
-		if(typical.trajectory) {
-			traj.idx <- bayesTFR:::get.typical.trajectory.index(traj)
-		} else {
-			thintraj <- bayesTFR:::get.thinning.index(nr.traj, dim(traj)[2])
-			if (thintraj$nr.points > 0) 
-		 		traj.idx <- thintraj$index
-		}
-	}
-	if(!is.null(traj)) 
-	 	rownames(traj) <- pop.pred$proj.years
-	return(list(trajectories=traj, index=traj.idx, quantiles=quant, age.idx=age.idx, half.child=hch))
-}
-
-get.pop.trajectories.multiple.age <- function(pop.pred, country, sex=c('both', 'male', 'female'), 
-												age='all', nr.traj=NULL, proportion=FALSE) {
-	# Like get.pop.trajectories() but it doesn't sum up over ages and it doesn't return quantiles
-	# Called when creating pop pyramid. Doesn't handle potential support ratio.
-	traj.file <- file.path(pop.pred$output.dir, paste('totpop_country', country, '.rda', sep=''))
-	quant <- NULL
-	age.idx <- NULL
-	if (file.exists(traj.file)) {
-		load(traj.file)
-		sex <- match.arg(sex)
-		max.age <- dim(totp)[1] # should be 27
-		age.idx <- if(age[1]=='all') 1:max.age else age
-		if(sex == 'both') 
-			traj <- totpm[age.idx,,,drop=FALSE] + totpf[age.idx,,,drop=FALSE]
-		else {
-			traj <- if(sex=='male') totpm[age.idx,,,drop=FALSE] else totpf[age.idx,,,drop=FALSE]
-
-			if(proportion) {
-				totpop <- (apply(totpm[,,,drop=FALSE], c(2,3), sum) + apply(totpf[,,,drop=FALSE], c(2,3), sum))
-				for(iage in 1:dim(traj)[1])
-					traj[iage,,] <- traj[iage,,]/totpop
-			}
-		}
-		thintraj <- bayesTFR:::get.thinning.index(nr.traj, dim(traj)[3])
-		if (thintraj$nr.points == 0) return(list(trajectories=NULL))
-		traj.idx <- thintraj$index
-	} else {
-		traj <- NULL
-		traj.idx <- NULL
-	}	
-	if(!is.null(traj)) 
-	 	dimnames(traj)[[2]] <- pop.pred$proj.years
-	return(list(trajectories=traj, index=traj.idx, age.idx=age.idx))
-}
-
-is.saved.pi <- function(pop.pred, pi, warning=TRUE) {
-	if(length(pi) == 0) return(NULL)
-	is.valid.pi <- rep(NA, length(pi))
-	quantile.values <- as.numeric(dimnames(pop.pred$quantiles)[[2]])
-	for (i in 1:length(pi)) {
-		al <- 1-(1-pi[i]/100)/2		
-		is.valid.pi[i] <- any(round(quantile.values,6)==round(al,6))
-		if(!is.valid.pi[i] && warning)
-			warning(pi[i], '% interval not available.')
-	}
-	return(is.valid.pi)
-}
-
-get.pop.traj.quantiles <- function(quantile.array, pop.pred, country.index, country.code, 
-									trajectories=NULL, pi=80, q=NULL, reload=TRUE, ...) {
-	al <- if(!is.null(q)) q else c((1-pi/100)/2, 1-(1-pi/100)/2)
-	found <- FALSE
-	if(!is.null(quantile.array)) {
-		quantile.values <- as.numeric(dimnames(quantile.array)[[2]])
-		alidx<-round(quantile.values,6)==round(al[1],6)
-		cqp <- NULL
-		if (any(alidx)) { # pre-saved quantiles
-			cqp <- quantile.array[country.index, alidx,]
-			if(length(al) > 1) {
-				alidx2 <- round(quantile.values,6)==round(al[2],6)
-				cqp <- rbind(cqp, quantile.array[country.index, alidx2,])
-			}
-			found <- TRUE
-		} 
-	}
-	if(!found) { # non-standard quantiles
-		if(is.null(trajectories) && !reload) {
-			warning('Quantiles not found')
-			return(NULL)	
-		}
-		reload <- FALSE
-		if (is.null(trajectories)) {
-			if(pop.pred$nr.traj > 0) reload <- TRUE
-		} else { 
-			if (dim(trajectories)[2] < 2000 && pop.pred$nr.traj > dim(trajectories)[2]) reload <- TRUE
-		}
-		if(reload) {
-			#load 2000 trajectories maximum for computing quantiles
-			traj.reload <- get.pop.trajectories(pop.pred, country.code, nr.traj=2000, ...)
-			trajectories <- traj.reload$trajectories
-		}
-		if (!is.null(trajectories)) {
-			cqp <- apply(trajectories, 1, 
-						quantile, al, na.rm = TRUE)
-		} else {
-			warning('Quantiles not found')
-			return(NULL)
-		}
-	}
-	return(cqp)
-}
-
-get.age.labels <- function(ages, collapsed=FALSE) {
-	all.ages <- c(seq(0, by=5, length=27), NA)
-	ages.idx <- which(is.element(all.ages, ages))
-	ages.idx.shift <- ages.idx+1
-	if(collapsed) {
-		ages.idx.dif <- which(!is.element(ages.idx, ages.idx.shift))
-		ages.idx.shift <- ages.idx.shift[!is.element(ages.idx.shift, ages.idx)]
-		ages.idx <- ages.idx[ages.idx.dif]
-	}
-	lages <- all.ages[ages.idx]
-	uages <- all.ages[ages.idx.shift]
-	l <- length(lages)
-	result <- paste(all.ages[ages.idx[1:(l-1)]], '-', all.ages[ages.idx.shift[1:(l-1)]]-1, sep='')
-	if (l > 1) result <- c(result, if(is.na(all.ages[ages.idx.shift[l]])) paste(all.ages[ages.idx[l]], '+', sep='')
-			else paste(all.ages[ages.idx[l]], '-', all.ages[ages.idx.shift[l]]-1, sep=''))
-	return(result)
-}	
 
 pop.trajectories.plotAll <- function(pop.pred, 
 									output.dir=file.path(getwd(), 'pop.trajectories'),
-									output.type="png", verbose=FALSE, ...) {
+									output.type="png", expression=NULL, verbose=FALSE,  ...) {
 	# plots pop trajectories for all countries
-	if(!file.exists(output.dir)) dir.create(output.dir, recursive=TRUE)
-	all.countries <- pop.pred$countries[,'code']
-	postfix <- output.type
-	if(output.type=='postscript') postfix <- 'ps'
-	for (country in all.countries) {
-		country.obj <- get.country.object(country, country.table=pop.pred$countries)
-		if(verbose)
-			cat('Creating population graph for', country.obj$name, '(', country.obj$code, ')\n')
-
-		do.call(output.type, list(file.path(output.dir, 
-										paste('pop.plot_c', country.obj$code, '.', postfix, sep=''))))
-		pop.trajectories.plot(pop.pred, country=country.obj$code, ...)
-		dev.off()
-	}
-	if(verbose)
-		cat('\nTrajectory plots stored into', output.dir, '\n')
+	if(!is.null(expression) && !grepl('XXX', expression, fixed=TRUE))
+		stop('Expression must contain a mask "XXX" to be used as country code.')
+	bayesTFR:::.do.plot.all.country.loop(pop.pred$countries[,'code'], meta=NULL, country.table=pop.pred$countries,
+					output.dir=output.dir, func=.trajectories.plot.with.mask.replacement, output.type=output.type, 
+					file.prefix='pop.plot', plot.type='population graph', verbose=verbose, pop.pred=pop.pred, 
+					expression=expression, ...)
 }
 
+.trajectories.plot.with.mask.replacement <- function(pop.pred, country, expression=NULL, plotfun='pop.trajectories.plot', ...) {
+	expr.arg <- NULL
+	country.arg <- NULL
+	if(!is.null(expression))
+		expr.arg <- gsub('XXX', as.character(country), expression, fixed=TRUE)
+	else country.arg <- country
+	do.call(plotfun, list(pop.pred, country=country.arg, expression=expr.arg, ...))
+}
 
-pop.trajectories.plot <- function(pop.pred, country, pi=c(80, 95),
+pop.trajectories.plot <- function(pop.pred, country=NULL, expression=NULL, pi=c(80, 95),
 								  sex=c('both', 'male', 'female'), age='all',
 								  sum.over.ages=FALSE,
 								  half.child.variant=FALSE,
@@ -221,8 +33,8 @@ pop.trajectories.plot <- function(pop.pred, country, pi=c(80, 95),
 								  ) {
 	# lwd is a vector of 5 line widths for: 
 	#	1. observed data, 2. median, 3. quantiles, 4. half child variant, 5. trajectories
-	if (missing(country)) {
-		stop('Argument "country" must be given.')
+	if (is.null(country) && is.null(expression)) {
+		stop('Argument "country" or "expression" must be given.')
 	}
 	if(length(lwd) < 5) {
 		llwd <- length(lwd)
@@ -235,16 +47,19 @@ pop.trajectories.plot <- function(pop.pred, country, pi=c(80, 95),
 		col[(lcol+1):5] <- c('black', 'red', 'red', 'blue', 'gray')[(lcol+1):5]
 	}
 
-	country <- get.country.object(country, country.table=pop.pred$countries)
-	if(sum.over.ages || age[1]=='psr')
-		do.pop.trajectories.plot(pop.pred, country, pi=pi, sex=sex, age=age,
+	if(!is.null(country)) {
+		country <- get.country.object(country, country.table=pop.pred$countries)
+		if(is.null(country$code)) stop('Country not available.')
+	}
+	if(sum.over.ages || age[1]=='psr' || !is.null(expression))
+		do.pop.trajectories.plot(pop.pred, country, expression=expression, pi=pi, sex=sex, age=age,
 									half.child.variant=half.child.variant, nr.traj=nr.traj,
 									typical.trajectory=typical.trajectory,
 									xlim=xlim, ylim=ylim, xlab=xlab, ylab=ylab, main=main, lwd=lwd, col=col,
 									show.legend=show.legend, ann=ann, ...)
 	else {
 		all.ages <- pop.pred$ages
-		if(age=='all') age <- 1:20
+		if(age[1]=='all') age <- 1:20
 		age.labels <- get.age.labels(pop.pred$ages)
 		if(is.null(main)) {
 			main <- country$name
@@ -277,7 +92,7 @@ pop.trajectories.plot <- function(pop.pred, country, pi=c(80, 95),
 	}
 }
 
-do.pop.trajectories.plot <- function(pop.pred, country, pi=c(80, 95),
+do.pop.trajectories.plot <- function(pop.pred, country=NULL, expression=NULL, pi=c(80, 95),
 								  sex=c('both', 'male', 'female'), age='all',
 								  half.child.variant=FALSE,
 								  nr.traj=NULL, typical.trajectory=FALSE,
@@ -288,18 +103,26 @@ do.pop.trajectories.plot <- function(pop.pred, country, pi=c(80, 95),
 								  ) {
 
 	sex <- match.arg(sex)
-	trajectories <- get.pop.trajectories(pop.pred, country$code, sex, age, nr.traj, 
+	if(!is.null(expression)) {
+		trajectories <- get.pop.trajectories.from.expression(expression, pop.pred, nr.traj, 
 										typical.trajectory=typical.trajectory)
+		pop.observed.all <- get.pop.observed.from.expression(expression, pop.pred)
+	} else {
+		trajectories <- get.pop.trajectories(pop.pred, country$code, sex, age, nr.traj, 
+										typical.trajectory=typical.trajectory)
+		pop.observed.all <- get.pop.observed(pop.pred, country$code, sex=sex, age=age)
+	}
 	cqp <- list()
 	for (i in 1:length(pi))
 		cqp[[i]] <- get.pop.traj.quantiles(trajectories$quantiles, pop.pred, country$index, country$code, 
 										trajectories=trajectories$trajectories, pi=pi[i], sex=sex, age=age)
-	pop.observed <- get.pop.observed(pop.pred, country$code, sex=sex, age=age)
-	obs.not.na <- !is.na(pop.observed)
-	pop.observed <- if(sum(obs.not.na)==0) pop.observed[length(pop.observed)] else pop.observed[obs.not.na]
-	x1 <- as.integer(names(pop.observed))
-	x2 <- as.numeric(dimnames(pop.pred$quantiles)[[3]])
-	y1 <- pop.observed
+	
+	obs.not.na <- !is.na(pop.observed.all)
+	pop.observed.idx <- if(sum(obs.not.na)==0) length(pop.observed.all) else which(obs.not.na)
+	x1 <- as.integer(names(pop.observed.all)[pop.observed.idx])
+	x2 <- if(!is.null(dimnames(trajectories$trajectories))) as.numeric(dimnames(trajectories$trajectories)[[1]])
+			else as.numeric(dimnames(pop.pred$quantiles)[[3]])
+	y1 <- pop.observed.all[pop.observed.idx]
 	if(is.null(xlim)) xlim <- c(min(x1, x2), max(x1, x2))
 	if(is.null(ylim)) 
 		ylim <- c(min(y1, if (!is.null(trajectories$trajectories))
@@ -310,13 +133,16 @@ do.pop.trajectories.plot <- function(pop.pred, country, pi=c(80, 95),
 				  			trajectories$trajectories[,trajectories$index] else NULL, 
 				  		  sapply(cqp, max, na.rm=TRUE), na.rm=TRUE))
 	if(is.null(main)) {
-		main <- country$name 
-		if(sex != 'both') main <- paste(main, ': ', sex, sep='')
-		if(age[1] == 'psr') main <- paste(main, ' (Potential Support Ratio)', sep='')
+		if(!is.null(expression)) main <- expression
 		else {
-			if(age[1] != 'all') {
-				age.labels <- get.age.labels(pop.pred$ages[age], collapsed=TRUE)
-				main <- paste(main, ' (Age ', paste(age.labels, collapse=','), ')', sep='')
+			main <- country$name 
+			if(sex != 'both') main <- paste(main, ': ', sex, sep='')
+			if(age[1] == 'psr') main <- paste(main, ' (Potential Support Ratio)', sep='')
+			else {
+				if(age[1] != 'all') {
+					age.labels <- get.age.labels(pop.pred$ages[age], collapsed=TRUE)
+					main <- paste(main, ' (Age ', paste(age.labels, collapse=','), ')', sep='')
+				}
 			}
 		}
 	}
@@ -324,7 +150,7 @@ do.pop.trajectories.plot <- function(pop.pred, country, pi=c(80, 95),
 	plot(x1, y1, type=type, xlim=xlim, ylim=ylim, ylab=ylab, xlab=xlab, main=main, 
 			panel.first = grid(), lwd=lwd[1], col=col[1], ann=ann, ...)
 	# plot trajectories
-	if(!is.null(trajectories$trajectories)) {
+	if(!is.null(trajectories$index) && !is.null(trajectories$trajectories)) {
 		for (i in 1:length(trajectories$index)) {
 			lines(x2, trajectories$trajectories[,trajectories$index[i]], type='l', col=col[5], lwd=lwd[5])
 		}
@@ -366,18 +192,16 @@ do.pop.trajectories.plot <- function(pop.pred, country, pi=c(80, 95),
 }
 
 
-pop.trajectories.table <- function(pop.pred, country, pi=c(80, 95),
+pop.trajectories.table <- function(pop.pred, country=NULL, expression=NULL, pi=c(80, 95),
 								  sex=c('both', 'male', 'female'), age='all',
 								  half.child.variant=FALSE) {
-	if (missing(country)) {
-		stop('Argument "country" must be given.')
-	}
-	country <- get.country.object(country, country.table=pop.pred$countries)
-	max.age.idx <- 27
-	x <- pop.pred$proj.years
+	if (is.null(country)  && is.null(expression)) 
+		stop('Argument "country" or "expression" must be given.')
+		
+	if(!is.null(country))
+		country <- get.country.object(country, country.table=pop.pred$countries)
+	max.age.idx <- length(pop.pred$ages)
 	sex <- match.arg(sex)
-	l <- length(pop.pred$proj.years)
-	pred.table <- matrix(NA, ncol=2*length(pi)+1, nrow=l)
 	quant <- NULL
 	if (age[1]=='all') age.idx <- 1:max.age.idx
 	else {
@@ -385,41 +209,254 @@ pop.trajectories.table <- function(pop.pred, country, pi=c(80, 95),
 		else age.idx <- unique(age)
 	}
 	lage <- length(age.idx)
-	if(lage==max.age.idx) {
-		if(sex == 'both') quant <- pop.pred$quantiles
-		else quant <- if(sex=='male') pop.pred$quantilesM else pop.pred$quantilesF
+	if(!is.null(expression)) {
+		trajectories <- get.pop.trajectories.from.expression(expression, pop.pred)
+		pop.observed <- get.pop.observed.from.expression(expression, pop.pred)
+		quant <- NULL
+	} else {
+		trajectories <- list(trajectories=NULL)
+		pop.observed <- get.pop.observed(pop.pred, country$code, sex=sex, age=age.idx)
+		if(lage==max.age.idx) {
+			if(sex == 'both') quant <- pop.pred$quantiles
+			else quant <- if(sex=='male') pop.pred$quantilesM else pop.pred$quantilesF
+		}
 	}
-	pred.table[,1] <- get.pop.traj.quantiles(quant, pop.pred, country$index, country$code, 
-												q=0.5, sex=sex, age=age.idx)
+	x1 <- names(pop.observed)[-length(pop.observed)]
+	x2 <- if(!is.null(dimnames(trajectories$trajectories))) dimnames(trajectories$trajectories)[[1]]
+			else dimnames(pop.pred$quantiles)[[3]]
+	l <- length(x1) + length(x2)
+	pred.table <- matrix(NA, ncol=2*length(pi)+1, nrow=l)
+	rownames(pred.table) <- c(x1, x2)
+	pred.table[x1,1] <- pop.observed[-length(pop.observed)]
+	pred.table[x2,1] <- get.pop.traj.quantiles(quant, pop.pred, country$index, country$code, 
+								trajectories=trajectories$trajectories,	q=0.5, sex=sex, age=age.idx)
+	#if(is.na(pred.table[x2[1],1])) pred.table[x2[1],1] <- pop.observed[length(pop.observed)]
 	colnames(pred.table) <- c('median', rep(NA,ncol(pred.table)-1))
 	idx <- 2
 	for (i in 1:length(pi)) {
 		cqp <- get.pop.traj.quantiles(quant, pop.pred, country$index, country$code, 
-										pi=pi[i], sex=sex, age=age.idx)
+							trajectories=trajectories$trajectories, pi=pi[i], sex=sex, age=age.idx)
 		if (!is.null(cqp)) {
-			pred.table[,idx:(idx+1)] <- t(cqp)
+			pred.table[x2,idx:(idx+1)] <- t(cqp)
 		} else{
-			pred.table[,idx:(idx+1)] <- matrix(NA, nrow=l, ncol=2)
+			pred.table[x2,idx:(idx+1)] <- matrix(NA, nrow=l, ncol=2)
 		}
 		al <- (1-pi[i]/100)/2
 		colnames(pred.table)[idx:(idx+1)] <- c(al, 1-al)
 		idx <- idx+2
 	}
-	rownames(pred.table) <- x
 	cn <- colnames(pred.table)[2:ncol(pred.table)]
 	pred.table[,2:ncol(pred.table)] <- pred.table[,cn[order(cn)]]
 	colnames(pred.table)[2:ncol(pred.table)] <- cn[order(cn)]
-	if(half.child.variant) {
+	if(half.child.variant && is.null(expression)) {
 		# load the half child variants from trajectory file
 		traj <- get.pop.trajectories(pop.pred, country$code, sex, age, nr.traj=0)
 		if(!is.null(traj$half.child)) {
-			pred.table <- cbind(pred.table, traj$half.child)
+			pred.table <- cbind(pred.table, rbind(matrix(NA, nrow=length(x1), ncol=2), traj$half.child))
 			colnames(pred.table)[(ncol(pred.table)-1):ncol(pred.table)] <- c('-0.5child', '+0.5child')
 		}
 	}
 	return(pred.table)
 }
 
+get.data.byage <- function(pop.pred, country.object, year=NULL, expression=NULL, pi=c(80, 95),
+							sex=c('both', 'male', 'female'), nr.traj=NULL, typical.trajectory=FALSE, load.trajectories=TRUE) {
+	projection <- TRUE
+	projection.index <- 1
+	if(!is.null(year)) {
+		ind.proj <- get.predORobs.year.index(pop.pred, year)
+		projection.index <- ind.proj['index']
+		projection <- ind.proj['is.projection']
+		if(is.null(projection.index)) stop('Year ', year, ' not found.')
+	}
+	country <- country.object
+	cqp <- list()
+	trajectories <- list(trajectories=NULL)
+	last.open <- TRUE
+	if(projection) {
+		if(!is.null(expression)) {
+			trajectories <- get.pop.trajectories.from.expression.multiple.age(expression, pop.pred, nr.traj, 
+										typical.trajectory=typical.trajectory)
+			age.idx <- if(!is.null(rownames(trajectories$trajectories))) 
+							as.integer(rownames(trajectories$trajectories)) 
+						else 1:nrow(trajectories$trajectories)
+		} else {
+			if(load.trajectories) {
+				trajectories <- get.pop.trajectories.multiple.age(pop.pred, country$code, sex, nr.traj=nr.traj, 
+										typical.trajectory=typical.trajectory)
+				age.idx <- trajectories$age.idx
+			} else {
+				if(sex == 'male') trajectories$quantiles <- pop.pred$quantilesMage
+				else {
+					if(sex=='female') trajectories$quantiles <- pop.pred$quantilesFage
+					else # load trajectories to get quantiles 
+						trajectories <- get.pop.trajectories.multiple.age(pop.pred, country$code, sex)
+				}
+				age.idx<-1:27
+			}
+		}
+		pop.median <- get.pop.traj.quantiles.byage(trajectories$quantiles, pop.pred, country$index, country$code, 
+										projection.index, trajectories=trajectories$trajectories, q=0.5, sex=sex)
+		ylim.loc <- c(min(pop.median, na.rm=TRUE), max(pop.median, na.rm=TRUE))
+		if(!is.null(trajectories$trajectories)) 
+			ylim.loc <- c(min(ylim.loc[1], trajectories$trajectories[,projection.index,trajectories$index], na.rm=TRUE),
+							max(ylim.loc[2], trajectories$trajectories[,projection.index,trajectories$index], na.rm=TRUE))
+		for (i in 1:length(pi))
+			cqp[[i]] <- get.pop.traj.quantiles.byage(trajectories$quantiles, pop.pred, country$index, country$code, 
+										projection.index, trajectories=trajectories$trajectories, pi=pi[i], sex=sex)
+		if(length(pi) > 0)
+			ylim.loc <- c(min(ylim.loc[1], sapply(cqp, min, na.rm=TRUE), na.rm=TRUE),
+							max(ylim.loc[2], sapply(cqp, max, na.rm=TRUE), na.rm=TRUE))
+		year.label <- get.pop.prediction.periods(pop.pred)[projection.index]
+		if(length(age.idx)<27) last.open <- FALSE
+	} else { # historical year 
+		if(!is.null(expression)) {
+			pop.observed <- get.pop.observed.from.expression.multiple.age(expression, pop.pred)
+			pop.median <- pop.observed[,projection.index]
+			age.idx <- if(!is.null(rownames(pop.observed))) as.integer(rownames(pop.observed)) else 1:nrow(pop.observed)
+		} else {
+			pop.observed <- get.pop.observed.with.age(pop.pred, country$code, sex=sex)
+			pop.median <- pop.observed$data[,projection.index]
+			age.idx <- pop.observed$age.idx
+		}		
+		ylim.loc <- c(min(pop.median), max(pop.median))		
+		year.label <- get.pop.observed.periods(pop.pred)[projection.index]
+		if(length(age.idx)<21) last.open <- FALSE
+	}
+	age.labels <- get.age.labels(age.idx, age.is.index=TRUE, last.open=last.open)
+	return(list(trajectories=trajectories, age.idx=age.idx, age.labels=age.labels, year.label=year.label, ylim.loc=ylim.loc,
+					cqp=cqp, pop.median=pop.median, projection=projection, projection.index=projection.index))
+}
+
+pop.byage.plotAll <- function(pop.pred, 
+									output.dir=file.path(getwd(), 'pop.byage'),
+									output.type="png", expression=NULL, verbose=FALSE,  ...) {
+	# plots pop trajectories by age for all countries
+	if(!is.null(expression) && !grepl('XXX', expression, fixed=TRUE))
+		stop('Expression must contain a mask "XXX" to be used as country code.')
+	bayesTFR:::.do.plot.all.country.loop(pop.pred$countries[,'code'], meta=NULL, country.table=pop.pred$countries,
+					output.dir=output.dir, func=.trajectories.plot.with.mask.replacement, output.type=output.type, 
+					file.prefix='pop.byage.plot', plot.type='population graph', verbose=verbose, pop.pred=pop.pred, 
+					expression=expression, plotfun='pop.byage.plot', ...)
+}
+
+
+pop.byage.plot <- function(pop.pred, country=NULL, year=NULL, expression=NULL, pi=c(80, 95),
+								  sex=c('both', 'male', 'female'), 
+								  half.child.variant=FALSE,
+								  nr.traj=NULL, typical.trajectory=FALSE,
+								  xlim=NULL, ylim=NULL,  
+								  xlab='Age', ylab='Population projection', main=NULL, 
+								  lwd=c(2,2,2,1), col=c('red', 'red', 'blue', 'gray'),
+								  show.legend=TRUE, add=FALSE, ann=par('ann'), ...
+								  ) {
+
+	sex <- match.arg(sex)
+	if (is.null(country)  && is.null(expression)) 
+		stop('Argument "country" or "expression" must be given.')
+		
+	if(!is.null(country))
+		country <- get.country.object(country, country.table=pop.pred$countries)
+		
+	data <- get.data.byage(pop.pred, country, year, expression, pi=pi,
+							sex=sex, nr.traj=nr.traj, typical.trajectory=typical.trajectory)
+	with(data, {
+	x <- 1:length(age.idx)
+	y <- pop.median
+	if(is.null(xlim)) xlim <- c(min(x), max(x))
+	if(is.null(ylim)) ylim <- ylim.loc
+		
+	if(is.null(main)) {
+		if(!is.null(expression)) main <- paste(year.label, ':', expression)
+		else {
+			main <- paste(country$name, year.label) 
+			if(sex != 'both') main <- paste(main, ': ', sex, sep='')		
+		}
+	}
+	if(!add) {
+		plot(x, y, type='n', xlim=xlim, ylim=ylim, ylab=ylab, xlab=xlab, main=main, 
+			panel.first = grid(), ann=ann,  xaxt='n', ...)
+		if(ann) axis(1, at=1:length(age.idx), labels=age.labels, las=2)
+	}
+	# plot trajectories
+	if(!is.null(trajectories$index) && !is.null(trajectories$trajectories)) {
+		for (i in 1:length(trajectories$index)) {
+			lines(x, trajectories$trajectories[,projection.index,trajectories$index[i]], type='l', col=col[4], lwd=lwd[4])
+		}
+	}
+	# median
+	lines(x,y, lwd=lwd[1], col=col[1])
+	# plot given CIs
+	if(projection) {
+		lty <- 2:(length(pi)+1)
+		for (i in 1:length(pi)) {		
+			if (!is.null(cqp[[i]])) {
+				lines(x, cqp[[i]][1,], type='l', col=col[2], lty=lty[i], lwd=lwd[2])
+				lines(x, cqp[[i]][2,], type='l', col=col[2], lty=lty[i], lwd=lwd[2])
+			}
+		}
+		legend <- c('median', paste(pi, '% PI', sep=''))
+	} else {
+		legend <- c('observed')
+		lty <- c()
+		pi <- c()
+	}
+	lty <- c(1, lty)
+	lwds <- c(lwd[1], rep(lwd[2], length(pi)))
+	cols <- c(col[1], rep(col[2], length(pi)))
+	if (half.child.variant && !is.null(trajectories$half.child)) {
+		lty <- c(lty, max(lty)+1)
+		llty <- length(lty)
+		lines(x, trajectories$half.child[,projection.index,1], type='l', col=col[3], lty=lty[llty], lwd=lwd[3])
+		lines(x, trajectories$half.child[,projection.index,2], type='l', col=col[3], lty=lty[llty], lwd=lwd[3])
+		legend <- c(legend, '+/- 0.5 child')
+		cols <- c(cols, col[3])
+		lwds <- c(lwds, lwd[3])
+	}
+	if(show.legend && ann)
+		legend('topleft', legend=legend, lty=lty, bty='n', col=cols, lwd=lwds)
+	})
+}
+
+pop.byage.table <- function(pop.pred, country=NULL, year=NULL, expression=NULL, pi=c(80, 95),
+								  sex=c('both', 'male', 'female'), 
+								  half.child.variant=FALSE) {
+	sex <- match.arg(sex)
+	if (is.null(country)  && is.null(expression)) 
+		stop('Argument "country" or "expression" must be given.')
+		
+	if(!is.null(country))
+		country <- get.country.object(country, country.table=pop.pred$countries)
+		
+	data <- get.data.byage(pop.pred, country, year, expression, pi=pi,
+							sex=sex, load.trajectories= if(half.child.variant && is.null(expression)) TRUE else FALSE)
+	with(data, {
+	x <- age.labels
+	l <- length(x)
+	pred.table <- matrix(NA, ncol=1, nrow=l)
+	rownames(pred.table) <- x
+	pred.table[x,1] <- pop.median
+	colnames(pred.table) <- if(projection) paste('median', year.label, sep='.') else year.label
+	if(projection && !is.null(cqp)) {
+		idx <- 2
+		for (i in 1:length(pi)) {
+			pred.table <- cbind(pred.table, t(cqp[[i]]))
+			al <- (1-pi[i]/100)/2
+			colnames(pred.table)[idx:(idx+1)] <- c(al, 1-al)
+			idx <- idx+2
+		}
+		cn <- colnames(pred.table)[2:ncol(pred.table)]
+		pred.table[,2:ncol(pred.table)] <- pred.table[,cn[order(cn)]]
+		colnames(pred.table)[2:ncol(pred.table)] <- cn[order(cn)]
+	}
+	if(projection && half.child.variant && is.null(expression) && !is.null(trajectories$half.child)) {
+		# load the half child variants from trajectory file
+		pred.table <- cbind(pred.table, trajectories$half.child[,projection.index,])
+		colnames(pred.table)[(ncol(pred.table)-1):ncol(pred.table)] <- c('-0.5child', '+0.5child')
+	}
+	return(pred.table)
+	})
+}
 
 "get.bPop.pyramid" <- function(data, ...) UseMethod("get.bPop.pyramid")
 
@@ -488,7 +525,7 @@ get.bPop.pyramid.bayesPop.prediction <- function(data, country, year=NULL, pi=c(
 	pop.pred <- data
 	country <- get.country.object(country, country.table=pop.pred$countries)
 	if(is.null(country$code)) stop('Country not found in the prediction object.')
-	year.idx <- if(is.null(year)) 1 else sapply(year, get.prediction.year.index, pred=pop.pred)
+	year.idx <- if(is.null(year)) 1 else sapply(year, get.prediction.year.index, pop.pred=pop.pred)
 	lyears <- length(year.idx)
 	draw.projection <- rep(TRUE, lyears)
 	draw.projection[is.na(year.idx)] <- FALSE # if years not found in the prediction years, these are probably observed years
@@ -496,7 +533,7 @@ get.bPop.pyramid.bayesPop.prediction <- function(data, country, year=NULL, pi=c(
 	pop.observed <- NULL
 	if(draw.observed) {
 		pop.observed <- get.pop.observed(pop.pred, country$code, sex='both')
-		year.idx[!draw.projection] <- sapply(year[!draw.projection], get.observed.year.index, pred=pop.pred)
+		year.idx[!draw.projection] <- sapply(year[!draw.projection], get.observed.year.index, pop.pred=pop.pred)
 		if(all(is.na(year.idx))) stop('Unable to find data for year ', year)
 		if(any(is.na(year.idx))) warning('Unable to find data for year ', year[is.na(year.idx)])
 	}
@@ -615,9 +652,8 @@ pop.pyramid.bayesPop.pyramid <- function(pop.object, main=NULL, show.legend=TRUE
 										col.pi = NULL, ann=par('ann'), axes=TRUE, grid=TRUE, 
 										cex.main=0.9, cex.sub=1, cex=1, cex.axis=1, ...) {
 	mgp <- par('mgp')
-	oma <- par('oma')
 	mar <- par('mar')
-	par(oma = c(0, 0, 2, 0), mgp=c(3,0.5,0))
+	par(mgp=c(3,0.5,0)) 
 	par(mar=c(5, 4, 2, 4) + 0.1)
 	if((is.null(pop.object$pyramid) || length(pop.object$pyramid) == 0) && is.null(pop.object$CI)) 
 		stop('Nothing to be plotted. Either pyramid or CI must be given in pop.object.')
@@ -706,7 +742,7 @@ pop.pyramid.bayesPop.pyramid <- function(pop.object, main=NULL, show.legend=TRUE
 		if(is.null(main)) main <- if(exists('label')) label else ""
 		if(ann) title(main, line=1, cex.main=cex.main)
 	})	
-	par(mgp=mgp, oma=oma, mar=mar)
+	par(mgp=mgp, mar=mar)
 }
 
 pop.pyramid.bayesPop.prediction <- function(pop.object, country, year=NULL, pi=c(80, 95), proportion=FALSE,
@@ -719,25 +755,27 @@ pop.pyramid.bayesPop.prediction <- function(pop.object, country, year=NULL, pi=c
 	invisible(data)
 }
 
-pop.pyramidAll <- function(pop.pred, year=NULL,
-									output.dir=file.path(getwd(), 'pop.pyramid'),
-									output.type="png", verbose=FALSE, ...) {
-	# plots pyramid for all countries
+.do.plot.pyramid.all <- function(pop.pred, output.dir, func, year=NULL, output.type="png", 
+						file.prefix='pyr', plot.type='pyramid(s)', 
+						main=NULL, verbose=FALSE, ...) {
 	if(!file.exists(output.dir)) dir.create(output.dir, recursive=TRUE)
 	all.countries <- pop.pred$countries[,'name']
 	postfix <- output.type
 	if(output.type=='postscript') postfix <- 'ps'
 	if(is.null(year)) year <- list(pop.pred$present.year)
 	if(!is.list(year)) year <- list(year)
+	main.arg <- main
 	for (country in all.countries) {
 		country.obj <- get.country.object(country, country.table=pop.pred$countries)
 		if(verbose)
-			cat('Creating pyramid(s) for', country, '(', country.obj$code, ')\n')
-
+			cat('Creating', plot.type, 'for', country, '(', country.obj$code, ')\n')
+		if(!is.null(main) && grepl('XXX', main, fixed=TRUE))
+			main.arg <- gsub('XXX', as.character(country.obj$name), main, fixed=TRUE)
 		for(y in year) {
 			do.call(output.type, list(file.path(output.dir, 
-										paste('pyr', paste(y, collapse='_'), '_c', country.obj$code, '.', postfix, sep=''))))
-			pop.pyramid(pop.pred, country=country.obj$code, year=y, ...)
+										paste(file.prefix, paste(y, collapse='_'), '_c', country.obj$code, '.', postfix, sep=''))))
+			do.call(func, list(pop.pred, country=country.obj$code, year=y, main=main.arg, ...))
+			pop.pyramid(pop.pred, country=country.obj$code, year=y, main=main.arg, ...)
 			dev.off()
 		}
 	}
@@ -746,6 +784,14 @@ pop.pyramidAll <- function(pop.pred, year=NULL,
 }
 
 
+pop.pyramidAll <- function(pop.pred, year=NULL,
+									output.dir=file.path(getwd(), 'pop.pyramid'),
+									output.type="png", verbose=FALSE, ...) {
+	# plots pyramid for all countries
+	.do.plot.pyramid.all(pop.pred, output.dir, pop.pyramid, year=year, output.type=output.type, verbose=verbose, ...)
+}
+
+
 "pop.trajectories.pyramid" <- function(pop.object, ...) UseMethod("pop.trajectories.pyramid")
 
 pop.trajectories.pyramid.bayesPop.prediction <- function(pop.object, country, year=NULL, pi=c(80, 95), 
@@ -867,46 +913,10 @@ pop.trajectories.pyramidAll <- function(pop.pred, year=NULL,
 									output.dir=file.path(getwd(), 'pop.traj.pyramid'),
 									output.type="png", verbose=FALSE, ...) {
 	# plots pyramid for all countries and all years given by 'year'
-	if(!file.exists(output.dir)) dir.create(output.dir, recursive=TRUE)
-	all.countries <- pop.pred$countries[,'name']
-	postfix <- output.type
-	if(output.type=='postscript') postfix <- 'ps'
-	if(is.null(year)) year <- list(pop.pred$present.year)
-	if(!is.list(year)) year <- list(year)
-	for (country in all.countries) {
-		country.obj <- get.country.object(country, country.table=pop.pred$countries)
-		if(verbose)
-			cat('Creating trajectory pyramid(s) for', country, '(', country.obj$code, ')\n')
-
-		for(y in year) {
-			do.call(output.type, list(file.path(output.dir, 
-										paste('pyr', paste(y, collapse='_'), '_c', country.obj$code, '.', postfix, sep=''))))
-			pop.trajectories.pyramid(pop.pred, country=country.obj$code, year=y, ...)
-			dev.off()
-		}
-	}
-	if(verbose)
-		cat('\nTrajectory pyramids stored into', output.dir, '\n')
+	.do.plot.pyramid.all(pop.pred, output.dir, pop.trajectories.pyramid, year=year, output.type=output.type, 
+					plot.type='trajectory pyramid(s)', verbose=verbose, ...)
 }
 
-.get.year.index <- function(year, years) {
-	lyears <- length(years)
-	res <- as.integer(cut(year, labels=1:lyears, breaks=c(years-3, years[lyears]+2)))
-	return(res)
-	
-	#breaks <- c(years-3, years[lyears]+2)
-	#h <- try(hist(year, breaks=breaks, plot=FALSE)$count, silent=TRUE)
-	#return(if(inherits(h, "try-error")) NULL else which(h > 0)[1])
-}
-get.prediction.year.index <- function(pred, year) {
-	years <- pred$proj.years
-	return(.get.year.index(year, years))
-}
-
-get.observed.year.index <- function(pred, year) {
-	years <- as.integer(colnames(pred$inputs$pop.matrix[['male']]))
-	return(.get.year.index(year, years))
-}
 
 .get.xtick.labels.for.pyramid <- function(maxx, proportion){
 	if(proportion) {
@@ -920,4 +930,102 @@ get.observed.year.index <- function(pred, year) {
 		return(round(seq(0, maxx, length=nticks), dec))
 	}
 	return(round(signif(seq(0, maxx, length=min(11,maxx)),2)))
-}
\ No newline at end of file
+}
+
+
+get.data.for.worldmap.bayesPop.prediction <- function(pred, quantile=0.5, year=NULL,
+							projection.index=1, pi=NULL, sex=c('both', 'male', 'female'), age='all', expression=NULL, ...) {
+	quantiles <- quantile
+	if (!is.null(pi)) {
+		qlower <- (1-pi/100)/2
+		quantiles <- c(quantile, qlower, 1-qlower)
+	}
+	if(!all(is.element(as.character(quantiles), as.character(get.quantiles.to.keep()))))
+		stop('The quantile and pi arguments must correspond to the following quantiles:\n', get.quantiles.to.keep())
+	sex <- match.arg(sex)
+	projection <- TRUE
+	if(!is.null(year)) {
+		ind.proj <- get.predORobs.year.index(pred, year)
+		projection.index <- ind.proj['index']
+		projection <- ind.proj['is.projection']
+		if(is.null(projection.index)) stop('Projection year ', year, ' not found.')
+	}
+	if(projection) {
+		if(!is.null(expression)) {
+			data <- get.pop.from.expression.all.countries(expression, pred, quantiles, projection.index)
+		} else {
+			if(!all(is.element(as.character(quantiles), dimnames(pred$quantiles)[[2]])))
+				stop('Some of the quantiles ', paste(quantiles, collapse=', '), ' not found.\nAvailable: ', 
+							paste(dimnames(pred$quantiles)[[2]], collapse=', '), 
+					 '\nCheck arguments "quantile" and "pi".')
+			data <- get.pop.all.countries(pred, quantiles, projection.index, sex, age)
+			if(is.null(data)) stop('Maps are not supported for the given combination of sex and age.')
+		}
+		period <- get.pop.prediction.periods(pred)[projection.index]
+	} else { # observed data
+		data <- if(!is.null(expression))
+					get.pop.observed.from.expression.all.countries(expression, pred, projection.index)
+				else 
+					get.pop.observed.all.countries(pred, projection.index, sex=sex, age=age)
+		period <- get.pop.observed.periods(pred)[projection.index]
+	}
+	rownames(data) <- NULL
+	low<-NULL
+	up<-NULL
+	res <- data
+	if(!is.null(dim(data))) {		
+		res <- data[,1]
+		if(ncol(data) > 1) {
+			low <- data[,2]
+			up <- data[,3]
+		}
+	}
+	return(list(period=period, data=res, country.codes=pred$countries$code, lower=low, upper=up, 
+				sex=sex, age=age, expression=expression))
+}
+
+
+pop.map <- function(pred, sex=c('both', 'male', 'female'), age='all', expression=NULL, ...) {
+	return(bayesTFR:::tfr.map(pred, par.name=expression, data.args=list(sex=sex, age=age, expression=expression), ...))
+}
+
+.map.main.default.bayesPop.prediction <- function(pred, dp, ...) {
+	if(!is.null(dp$expression)) main <- dp$expression
+	else {
+		main <- 'Pop'
+		if(dp$sex != 'both') main <- paste(main, dp$sex)
+		if(dp$age[1] != 'all') main <- paste(main, get.age.labels(dp$age, collapsed=TRUE, age.is.index=TRUE))
+	}
+	return(paste(main,': quantile', sep=''))
+}
+
+get.pop.map.parameters <- function(pred, expression=NULL, sex=c('both', 'male', 'female'), age='all', 
+								range=NULL, nr.cats=50, same.scale=TRUE, quantile=0.5, ...) {
+	map.pars <- list(pred=pred, quantile=quantile, expression=expression, sex=sex, age=age, ...)
+	if (same.scale) {
+		if(is.null(range)) {
+			if(!all(is.element(quantile, get.quantiles.to.keep()))) 
+			stop('The quantile and pi arguments must correspond to the following quantiles:\n', get.quantiles.to.keep())
+			sex <- match.arg(sex)
+			data <- if(!is.null(expression)) get.pop.from.expression.all.countries(expression, pred, quantile, 1)
+					else get.pop.all.countries(pred, quantile, 1, sex=sex, age=age)	
+			range <- c(min(data), max(data))
+		}
+		quant.values <- seq(range[1], range[2], length=nr.cats)
+		col <- rainbow(500, start=0, end=0.67)[seq(500, 1, length=length(quant.values)-1)]
+		map.pars$catMethod <- quant.values
+	} else {
+		col <- rainbow(500, start=0, end=0.67)[seq(500, 1, length=nr.cats)]
+		map.pars$numCats <- nr.cats
+	}
+	map.pars$colourPalette <- col
+	return(map.pars)		
+}
+
+pop.map.gvis <- function(pred, ...)
+	bdem.map.gvis(pred, ...)
+						
+bdem.map.gvis.bayesPop.prediction <- function(pred,  ...) {
+	bayesTFR:::.do.gvis.bdem.map('pop', 'Population', pred, ...)
+}
+
